﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net;
using System.IO;
using System.Xml.Serialization;
using System.Xml;
using System.Threading;

namespace MediaFrameRestApiDemo
{
    class Program
    {
        static void Main(string[] args)
        {
            if (args.Length < 1)
            {
                Console.WriteLine("Must provide server name on command line.");
                return;
            }

            MediaFrameRestApiDemo demo = new MediaFrameRestApiDemo(args[0]);
            demo.Run();
        }
    }

    class MediaFrameRestApiDemo
    {
        private string _serverName;
        private string _servicePath = "/mediaframe";

        private object SendRequest(string verb, string url, Type responseType)
        {
            return SendRequest(verb, url, responseType, null);
        }

        private object SendRequest(string verb, string url, Type responseType, object requestObj)
        {
            try
            {
                string fullUrl = "http://" + _serverName + _servicePath + url;
                HttpWebRequest request = (HttpWebRequest)HttpWebRequest.Create(fullUrl);
                request.Method = verb;
                request.KeepAlive = true;
                request.ContentType = "application/xml";
                
                if (requestObj != null)
                {
                    using (StreamWriter writer = new StreamWriter(request.GetRequestStream()))
                    {
                        
                        string requestXml = GetXmlFromObject(requestObj);
                        if (requestXml == null)
                        {
                            return null;
                        }
                        else
                        {
                            writer.Write(requestXml);
                            writer.Flush();
                        }
                    }
                }

                if (verb == "DELETE")
                    return string.Empty;
                else 
                {
                    HttpWebResponse response = (HttpWebResponse)request.GetResponse();
                    string strXml = new StreamReader(response.GetResponseStream()).ReadToEnd();
                    if (responseType != null)
                    {
                        XmlSerializer serializer = new XmlSerializer(responseType);
                        return serializer.Deserialize(new StringReader(strXml));
                    }
                    else
                    {
                        return strXml;
                    }
                }
            }

            catch (ProtocolViolationException pve)
            {
                Print("ProtocolViolationException: {0}\n{1}", pve.Message, pve.StackTrace);
            }
            catch (WebException we)
            {
                Print("WebException: {0}\n{1}", we.Message, we.StackTrace);
            }
            catch (InvalidOperationException ioe)
            {
                Print("InvalidOperationException: {0}\n{1}", ioe.Message, ioe.StackTrace);
            }
            catch (Exception e)
            {
                Print("Exception: {0}\n{1}", e.Message, e.StackTrace);
            }
            return null;
        }

        public string GetXmlFromObject(object obj)
        {
            String XmlizedString = null;
            MemoryStream memoryStream = new MemoryStream();
            XmlSerializer xs = new XmlSerializer(obj.GetType());
            XmlTextWriter xmlTextWriter = new XmlTextWriter(memoryStream, Encoding.UTF8);

            xs.Serialize(xmlTextWriter, obj);
            memoryStream = (MemoryStream)xmlTextWriter.BaseStream;
            XmlizedString = UTF8ByteArrayToString(memoryStream.ToArray());
            XmlizedString = XmlizedString.Trim();
            return XmlizedString;
        }

        private String UTF8ByteArrayToString(Byte[] characters)
        {
            UTF8Encoding encoding = new UTF8Encoding();
            String constructedString = encoding.GetString(characters);
            return (constructedString);
        }

        private void Print(string text, params object[] args)
        {
            Console.WriteLine(text, args);
        }

        public MediaFrameRestApiDemo(string serverName)
        {
            _serverName = serverName;
        }

        public void Run()
        {
            // Check the status of the MediaFrame server.
            // Page 15 - #2
            ServiceState state = (ServiceState)SendRequest("GET", "/foundation.svc/status", typeof(ServiceState));
            Print("State: {0}\n", state);
            if (state != ServiceState.Online ) throw new Exception("Cannot continue, MediaFrame system is not online.");

            // Get the Domain
            // Page 14 - #1
            URI url = (URI)SendRequest("GET", "/foundation.svc/domain", typeof(URI));
            Print("Domain: {0}\n", url.Url.Domain);

            // Get MediaFrame Types
            // Page 16 - #3
            ArrayOfRegisteredType arrayType1 = (ArrayOfRegisteredType)SendRequest("GET", "/foundation.svc/types", typeof(ArrayOfRegisteredType));
            if (arrayType1 == null) throw new Exception("Unable to retrieve ArrayOfRegisteredType's RegisteredType(s).");
            foreach (RegisteredType rt in arrayType1.RegisteredType)
            {
                Print("arrayType1: {0}", rt); 
            }

            string path = string.Empty;
            // Create a new mailbox in order to receive event notifications.
            // Page 42 - #13
            Guid mailboxGuid = (Guid)SendRequest("GET", "/notification.svc/mailbox", typeof(Guid));
            Print("Created new mailbox, GUID={0}\n", mailboxGuid);

////////////////////////////////////////////////////////////////////////////////////////////
            // Get Metadata Fields
            // Page 34 - #13
            path = string.Format("/asset.svc/metadata/schema/fields");
            ArrayOfFieldDescription1 responseDescriptions = (ArrayOfFieldDescription1)SendRequest("GET", path, typeof(ArrayOfFieldDescription1));
            foreach (FieldDescription description in responseDescriptions.FieldDescription)
            {
                Print("\tFieldDescription, Datatype={0}", description.Datatype);
                if (description.Options != null)
                {
                    foreach (FieldOption opt in description.Options)
                    {
                        Print("\tFieldOption, ToString={0}, Value={1}", opt.ToString(), opt.Value);
                    } 
                }
            }

////////////////////////////////////////////////////////////////////////////////////////////
            // Subscribe for Metadata event notifications.
            // Page 43 - #14
            MetadataFilter filter = new MetadataFilter("Any");
            ArrayOfSubscriptionFilter subscriptions = new ArrayOfSubscriptionFilter();
            subscriptions.SubscriptionFilter = new SubscriptionFilter[] { filter };
            string endTime = DateTime.Now.AddMinutes(5).ToString();
            string subscribeRequestURL = string.Format("/notification.svc/subscription?mailbox={0}&end={1}", mailboxGuid, endTime);
            Guid subscriptionGuid = (Guid)SendRequest("POST", subscribeRequestURL, typeof(Guid), subscriptions);
            Print("Created new subscription, GUID={0}", subscriptionGuid);

////////////////////////////////////////////////////////////////////////////////////////////
            // Create a new MediaFrame logical asset
            // Page 23 - #7
            PropertyType prop1 = new PropertyType(ItemChoiceType.@string, "sourceID", "test");
            PropertyList properties = new PropertyList();
            properties.Property = new PropertyType[] { prop1 };
            LogicalAsset newAsset = (LogicalAsset)SendRequest("POST", "/asset.svc/metadata", typeof(LogicalAsset), properties);
            Print("Created new logical asset, URI={0}", newAsset.Uri);
            foreach (PropertyType property in newAsset.Fields)
            {
                Print("\tLogical asset, Key={0}, Value={1}", property.Key, property.Value);
            }

////////////////////////////////////////////////////////////////////////////////////////////
            // Update MF Asset Metadata
            // Page 21 - #6
            prop1 = new PropertyType(ItemChoiceType.@string, "sourceID", "test 2");
            properties = new PropertyList();
            properties.Property = new PropertyType[] { prop1 };
            path = string.Format("/asset.svc/metadata/{0}", newAsset.Uri.Url.AssetId);
            LogicalAsset updatedAsset = (LogicalAsset)SendRequest("PUT", path, typeof(LogicalAsset), properties);
            Print("Updated logical asset, URI={0}", updatedAsset.Uri);
            foreach (PropertyType property in updatedAsset.Fields)
            {
                Print("\tUpdated Logical asset, Key={0}, Value={1}", property.Key, property.Value);
            }

////////////////////////////////////////////////////////////////////////////////////////////
            // Query MF Assets based on Metadata
            // Page 25 - #8
            path = string.Format("/asset.svc/metadata?query=&filters=&max=100");
            ArrayOfLogicalAsset arrayLogicalAsset = (ArrayOfLogicalAsset)SendRequest("GET", path, typeof(ArrayOfLogicalAsset));
            Print("Query by keyword, ID={0}", arrayLogicalAsset.LogicalAsset.ToString());

            path = string.Format("/asset.svc/metadata?query=brad%20patterson%20allan;default&filters=&max=100");
            ArrayOfLogicalAsset arrayLogicalAsset2 = (ArrayOfLogicalAsset)SendRequest("GET", path, typeof(ArrayOfLogicalAsset));
            Print("Query by keyword, ID={0}", arrayLogicalAsset2.LogicalAsset.ToString());

            path = string.Format("/asset.svc/metadata?query=\"test%202\";default&filters=&max=1");
            ArrayOfLogicalAsset arrayLogicalAsset3 = (ArrayOfLogicalAsset)SendRequest("GET", path, typeof(ArrayOfLogicalAsset));
            Print("Query by keyword, ID={0}", arrayLogicalAsset3.LogicalAsset.ToString());

////////////////////////////////////////////////////////////////////////////////////////////
            // Get MF Assets
            // Page 20 - #5
            string getMfAssetUri = string.Format("id={0}/K2-MDI/V:/winkerSD/banjo-sd1-20100402(232),video/k2.cmf", url.Url.Domain);
            path = string.Format("/asset.svc/related?{0}", getMfAssetUri);
            ArrayOfURN arrayOfUrn = (ArrayOfURN)SendRequest("GET", path, typeof(ArrayOfURN));
            if(arrayOfUrn.URN != null)
                Print("URN: {0}", arrayOfUrn.URN.ToString());

////////////////////////////////////////////////////////////////////////////////////////////
            // Get Asset Associations
            // Page 18 - #4
            path = string.Format("/asset.svc/associations/{0}", newAsset.Uri.Url.AssetId);
            ArrayOfTemporalURI turi = (ArrayOfTemporalURI)SendRequest("GET", path, typeof(ArrayOfTemporalURI));
            Print("State: {0}", ((TemporalURI)turi.TemporalURI[0]).State.ToString());

////////////////////////////////////////////////////////////////////////////////////////////
            // Create a new marker (keyword) for the new asset
            // Page 31 - #11
            Keyword newKeyword = new Keyword("", "test keyword phrase", "00:00:00", 1, "00:05:00", "00:00:00;00", "00:00:00;00", "");
            path = string.Format("/asset.svc/metadata/{0}/markers", newAsset.Uri.Url.AssetId);
            Keyword keywordResponse = (Keyword)SendRequest("POST", path, typeof(Keyword), newKeyword);
            Print("Created new keyword, ID={0}", keywordResponse.Id);

////////////////////////////////////////////////////////////////////////////////////////////
            // Get Asset Markers
            // Page 28 - #9
            path = string.Format("/asset.svc/metadata/{0}/markers", newAsset.Uri.Url.AssetId);
            ArrayOfKeyword arrayOfKeywordResponse = (ArrayOfKeyword)SendRequest("GET", path, typeof(ArrayOfKeyword));
            Print("Get asset Markers, ID={0}", arrayOfKeywordResponse.Keyword);

////////////////////////////////////////////////////////////////////////////////////////////
            // Update existing Asset Marker
            // Page 29 - #10
            newKeyword.Id = arrayOfKeywordResponse.Keyword[0].Id;
            path = string.Format("/asset.svc/metadata/markers/{0}", newKeyword.Id);
            newKeyword.Phrase = "Brad here.";
            Keyword updateKeywordResponse = (Keyword)SendRequest("PUT", path, typeof(Keyword), newKeyword);
            Print("Update asset Marker, ID={0}", updateKeywordResponse.Id);

////////////////////////////////////////////////////////////////////////////////////////////
            // Delete Asset Marker
            // Page 33 - #12
            newKeyword.Id = updateKeywordResponse.Id;
            path = string.Format("/asset.svc/metadata/markers/{0}", newKeyword.Id);
            object deleteKeywordResponse = (object)SendRequest("DELETE", path, typeof(object));
            //Print("Deleted asset Marker, ID={0}", deleteKeywordResponse.Id);

////////////////////////////////////////////////////////////////////////////////////////////
            // Renew Subscription
            // Page 51 - #17
            path = string.Format("/notification.svc/subscription/{0}", subscriptionGuid);
            DateTime newDate = new DateTime(2011, 4, 17);
            DateTime renewedSunscription = (DateTime)SendRequest("PUT", path, typeof(DateTime), newDate);
            
////////////////////////////////////////////////////////////////////////////////////////////
            // There can be a slight delay before notifications become available. Delay for a while
            // to make sure we see notificaitons for the two events above.
            Print("Waiting before polling for notifications...");
            Thread.Sleep(10000);
////////////////////////////////////////////////////////////////////////////////////////////
            // Poll for and display notifications.  This should show a Metadata creation event, and a Metadata modified event.
            // Page 45 - #15
            string pollRequestURL = string.Format("/notification.svc/subscription/{0}", subscriptionGuid);
            ArrayOfMediaFrameEvent events = (ArrayOfMediaFrameEvent)SendRequest("GET", pollRequestURL, typeof(ArrayOfMediaFrameEvent));
            Print("Event notifications:");
            if (events != null && events.MediaFrameEvent != null)
            {
                foreach (MediaFrameEvent ev in events.MediaFrameEvent)
                {
                    Print("  {0}", ev.ToString());
                }
            }
            else
            {
                Print("   No notifications available.");
            }

////////////////////////////////////////////////////////////////////////////////////////////
            // Unsubscribe from notifications.
            // Page 50 - #16
            Print("Unsubscribing...");
            string unsubscribeRequestURL = string.Format("/notification.svc/{0}", mailboxGuid);
            SendRequest("DELETE", unsubscribeRequestURL, null);




        }
    }
}
